@@ -163,16 +163,6 @@ Dcmtk_rt_study::image_load ()
         return;
     }
 
-    float rescale_slope, rescale_intercept;
-    rc = df->get_ds_float (DCM_RescaleIntercept, &rescale_intercept);
-    if (!rc) {
-        rescale_intercept = 0;
-    }
-    rc = df->get_ds_float (DCM_RescaleSlope, &rescale_slope);
-    if (!rc) {
-        rescale_slope = 1;
-    }
-
 #if defined (commentout)
     lprintf ("Samp_per_pix: %d\n", (int) samp_per_pix);
     lprintf ("Phot_interp: %s\n", phot_interp);
@@ -180,7 +170,6 @@ Dcmtk_rt_study::image_load ()
     lprintf ("Bits_stored: %d\n", (int) bits_stored);
     lprintf ("High_bit: %d\n", (int) high_bit);
     lprintf ("Pixel_rep: %d\n", (int) pixel_rep);
-    lprintf ("S/I = %f/%f\n", rescale_slope, rescale_intercept);
 #endif
     
     /* Some kinds of images we don't know how to deal with.  
@@ -442,14 +431,34 @@ Dcmtk_rt_study::image_load ()
 		"(%d vs. %d x %d).\n", length, dim[0], dim[1]);
 	}
 
+        float rescale_slope, rescale_intercept;
+        rc = df->get_ds_float (DCM_RescaleIntercept, &rescale_intercept);
+        if (!rc) {
+            rescale_intercept = 0;
+        }
+        rc = df->get_ds_float (DCM_RescaleSlope, &rescale_slope);
+        if (!rc) {
+            rescale_slope = 1;
+        }
+
+#if defined (commentout)
+        lprintf ("S/I = %f/%f\n", rescale_slope, rescale_intercept);
+#endif
         /* Apply slope and offset */
-        if (bits_alloc == 8) {
-            for (plm_long j = 0; j < (plm_long) length; j++) {
+        for (plm_long j = 0; j < (plm_long) length; j++) {
+            if (bits_alloc == 8 && pixel_rep == 0) {
+                img[j] = rescale_slope * (uint8_t) pixel_data_8[j] 
+                    + rescale_intercept;
+            }
+            else if (bits_alloc == 8 && pixel_rep == 1) {
                 img[j] = rescale_slope * (int8_t) pixel_data_8[j] 
                     + rescale_intercept;
             }
-        } else if (bits_alloc == 16) {
-            for (plm_long j = 0; j < (plm_long) length; j++) {
+            else if (bits_alloc == 16 && pixel_rep == 0) {
+                img[j] = rescale_slope * (uint16_t) pixel_data_16[j] 
+                    + rescale_intercept;
+            }
+            else if (bits_alloc == 16 && pixel_rep == 1) {
                 img[j] = rescale_slope * (int16_t) pixel_data_16[j] 
                     + rescale_intercept;
             }
@@ -505,6 +514,9 @@ dcmtk_save_slice (const Rt_study_metadata::Pointer rsm, Dcmtk_slice_data *dsd)
     /* XVI 4.5 requires a DCM_PositionReferenceIndicator */
     dataset->putAndInsertString (DCM_PositionReferenceIndicator, "SP");
 
+    /* General equipment module */
+    Dcmtk_module::set_general_equipment (dataset, image_metadata);
+
     /* General Image module */
     tmp = string_format ("%d", dsd->instance_no);
     dataset->putAndInsertString (DCM_InstanceNumber, tmp.c_str());
@@ -639,11 +651,15 @@ Dcmtk_rt_study::image_save (
             all_integers = false;
         }
     }
+        if (vol_max - vol_min > UINT16_MAX) {
+            all_integers = false;
+        }
     /* Use a heuristic to determine intercept and offset.
        The heuristic is designed around the following principles:
        - prevent underflow when using low-precision DICOM string-encoded
          floating point numbers
        - map integers to integers
+       - keep integer values within range
     */
     if (meta_intercept != "") {
         int rc = sscanf (meta_intercept.c_str(), "%f", &dsd.intercept);
@@ -661,6 +677,9 @@ Dcmtk_rt_study::image_save (
     }
     else if (all_integers) {
         dsd.slope = 1;
+        if (vol_max - vol_min > INT16_MAX) {
+            dsd.intercept = vol_max - INT16_MAX;
+        }
     }
     else {
         float range = vol_max - dsd.intercept;
