@@ -153,21 +153,27 @@ Segmentation::load_prefix (const char *prefix_dir)
         /* Look at filename, make sure it is an mha or nrrd file */
         const char *entry = dl.entries[i];
 	if (!extension_is (entry, ".mha") 
+            && !extension_is (entry, ".mhd")
+            && !extension_is (entry, ".nii")
+            && !extension_is (entry, ".nii.gz")
             && !extension_is (entry, ".nrrd"))
         {
             continue;
         }
 
-        /* Grab the structure name from the filename */
-        char *structure_name = strdup (entry);
-        strip_extension (structure_name);
-        lprintf ("Loading structure: %s\n", structure_name);
-
         /* Load the file */
         std::string input_fn = string_format ("%s/%s", prefix_dir, entry);
         Plm_image img (input_fn, PLM_IMG_TYPE_ITK_UCHAR);
         Plm_image_header pih (img);
 
+        /* Grab the structure name from the filename */
+        char *structure_name = strdup (entry);
+        if (extension_is (structure_name, "nii.gz")) {
+            strip_extension (structure_name);
+        }
+        strip_extension (structure_name);
+        lprintf ("Loading structure: %s\n", structure_name);
+
         if (first) {
             this->initialize_ss_image (pih, out_vec_len);
 
@@ -232,8 +238,12 @@ Segmentation::load_prefix (const char *prefix_dir)
     if (d_ptr->m_rtss) {
         d_ptr->m_rtss->free_all_polylines ();
     }
-    d_ptr->m_rtss_valid = false;
-    d_ptr->m_ss_img_valid = true;
+    if (first) {
+        /* We didn't get any valid images in this directory, so do nothing */
+    } else {
+        d_ptr->m_rtss_valid = false;
+        d_ptr->m_ss_img_valid = true;
+    }
 }
 
 void
@@ -468,6 +478,120 @@ Segmentation::save_labelmap (const std::string& labelmap_fn)
     d_ptr->m_labelmap->save_image (labelmap_fn);
 }
 
+unsigned int
+get_opt4d_flattened_index_from_3d_indices(itk::Index<3> index, itk::Size<3> image_size) {
+    auto col = index[0];
+    auto row = index[1];
+    auto slice = index[2];
+    auto ncols = image_size[0];
+    auto nrows = image_size[1];
+    auto nslices = image_size[2];
+    return col + slice * ncols + row * ncols * nslices;
+}
+
+void
+Segmentation::save_opt4d(const std::string &opt4d_prefix) {
+    auto file_name = opt4d_prefix + ".voi";
+    std::ofstream voi_file(file_name, std::ios::binary);
+    if (!voi_file.is_open()) {
+        throw std::runtime_error("Could not open output .voi file for writing.");
+    }
+
+    // Initialize the entire volume as air
+    auto dims = d_ptr->m_rtss->m_dim;
+    auto num_structures = d_ptr->m_rtss->num_structures;
+    auto num_voxels = dims[0] * dims[1] * dims[2];
+    std::vector<uint8_t> voi(num_voxels, 127);
+
+    std::vector<std::vector<unsigned int>> structure_indices_list(num_structures);
+
+    for (size_t i = 0; i < num_structures; i++) {
+        // Extract an ITK image for the structure
+        Rtss_roi *curr_structure = d_ptr->m_rtss->slist[i];
+        int bit = curr_structure->bit;
+        if (bit == -1) continue;
+        UCharImageType::Pointer prefix_img
+                = ss_img_extract_bit(d_ptr->m_ss_img, bit);
+
+        // Get indices which belong to the structure
+        itk::ImageRegionConstIteratorWithIndex<UCharImageType> it(prefix_img, prefix_img->GetLargestPossibleRegion());
+        auto size = prefix_img->GetLargestPossibleRegion().GetSize();
+        std::vector<unsigned int> indices;
+        for (it.GoToBegin(); !it.IsAtEnd(); ++it) {
+            if ((unsigned int) it.Get() == 1) {
+                indices.push_back(get_opt4d_flattened_index_from_3d_indices(it.GetIndex(), size));
+            }
+        }
+        std::sort(indices.begin(), indices.end());
+        structure_indices_list[i] = indices;
+
+        // Set the voxels in the structure to the structure index
+        for (const auto index: indices) {
+            voi[index] = i;
+        }
+    }
+
+    // Write the voxels to file. Opt4D expects each voxel to be a single byte
+    for (auto i = 0; i < num_voxels; i++) {
+        voi_file.write((char *) &voi[i], sizeof(char));
+    }
+    voi_file.close();
+
+
+    // Now let's try to write the .vv file
+    std::string opt4d_vv_fn = opt4d_prefix + ".vv";
+    std::ofstream vv_file(opt4d_vv_fn, std::ios::binary);
+    if (!vv_file.is_open()) {
+        throw std::runtime_error("Could not open output file");
+    }
+
+    // Write the .vv file header
+    std::string title = "vv-opt4D";
+    // When written as binary, this magic number will be "ab" or "ba" depending
+    // on system endianness
+    short magic_number = 25185;
+    char array[2];
+    *((short *) array) = magic_number;
+    std::string header_line_2 = "#VOIs:";
+
+    vv_file << title << array << "\n";
+
+    auto num_structures_string = std::to_string(num_structures);
+    vv_file << header_line_2 << std::setfill('0') << std::setw(5) << num_structures_string << "\n";
+
+    for (size_t i = 0; i < d_ptr->m_rtss->num_structures; i++) {
+        vv_file << std::setfill('0') << std::setw(5)
+                << std::to_string(i)
+                << " "
+                << d_ptr->m_rtss->get_structure_name(i)
+                << "\n";
+    }
+
+    // Use run-length encoding to write the vv file
+    for (const auto &indices: structure_indices_list) {
+        auto num_voxels = indices.size();
+        vv_file.write((char*)(&num_voxels), sizeof(int));
+        if (indices.empty()) continue;
+
+        std::vector<unsigned int>::const_iterator iter = indices.begin();
+        while (iter != indices.end()) {
+
+            unsigned int range_start = *iter;
+            unsigned char range_length = 0;
+
+            do {
+                ++range_length;
+                ++iter;
+            } while ((iter != indices.end()) && (*iter == (range_start + range_length)) && (range_length < 0xFF));
+            vv_file.write((char*)(&range_start), sizeof(range_start));
+            vv_file.write((char*)(&range_length), sizeof(range_length));
+        }
+    }
+    vv_file.close();
+}
+
+
+
 void
 Segmentation::save_prefix (const std::string &output_prefix,
     const std::string& extension)
