@@ -81,6 +81,9 @@ parse_fn (
         "a single structure", 1, "");
     parser->add_long_option ("", "output-colormap", 
         "create a colormap file that can be used with 3d slicer", 1, "");
+    parser->add_long_option ("", "output-opt4d", 
+        "create output files file.vv and file.voi that can be used "
+        "with opt4D; file extensions will be added automatically", 1, "");
     parser->add_long_option ("", "output-pointset", 
         "create a pointset file that can be used with 3d slicer", 1, "");
     parser->add_long_option ("", "output-prefix", 
@@ -94,6 +97,9 @@ parse_fn (
     parser->add_long_option ("", "output-ss-list", 
         "create a structure set list file containing names and colors", 
         1, "");
+    parser->add_long_option ("", "output-study",
+        "create a directory of nrrd files containing image, dose, and structures",
+        1, "");
     parser->add_long_option ("", "output-vf", 
         "create a vector field from the input xf", 1, "");
     parser->add_long_option ("", "output-xio", 
@@ -232,11 +238,13 @@ parse_fn (
     parms->output_labelmap_fn = parser->get_string("output-labelmap").c_str();
     parms->output_colormap_fn = parser->get_string("output-colormap").c_str();
     parms->output_pointset_fn = parser->get_string("output-pointset").c_str();
+    parms->output_opt4d_fn = parser->get_string("output-opt4d").c_str();
     parms->output_prefix = parser->get_string("output-prefix").c_str();
     parms->output_prefix_fcsv 
         = parser->get_string("output-prefix-fcsv").c_str();
     parms->output_ss_img_fn = parser->get_string("output-ss-img");
     parms->output_ss_list_fn = parser->get_string("output-ss-list");
+    parms->output_study_dirname = parser->get_string("output-study");
     parms->output_vf_fn = parser->get_string("output-vf").c_str();
     parms->output_xio_dirname = parser->get_string("output-xio").c_str();
     
@@ -430,15 +438,15 @@ do_command_warp (int argc, char* argv[])
         }
     }
 
-    /* What is the input file type? */
-    file_type = plm_file_format_deduce (parms.input_fn.c_str());
-
     /* Pointsets are a special case */
-    if (file_type == PLM_FILE_FMT_POINTSET) {
+    if (parms.output_pointset_fn != "") {
         warp_pointset_main (&parms);
         return;
     }
 
+    /* What is the input file type? */
+    file_type = plm_file_format_deduce (parms.input_fn.c_str());
+
     /* Process warp */
     rt_study_warp (&rt_study, file_type, &parms);
 
