@@ -485,26 +485,83 @@ Registration_parms::set_key_value (
     else if (key == "regularization")
     {
         if (!section_stage) goto key_only_allowed_in_section_stage;
+        Regularization_type& rtype
+            = stage->regularization_parms.regularization_type;
         if (val == "none") {
-            stage->regularization_type = REGULARIZATION_NONE;
+            rtype = REGULARIZATION_NONE;
         }
         else if (val == "analytic") {
-            stage->regularization_type = REGULARIZATION_BSPLINE_ANALYTIC;
+            rtype = REGULARIZATION_BSPLINE_ANALYTIC;
         }
         else if (val == "semi_analytic") {
-            stage->regularization_type = REGULARIZATION_BSPLINE_SEMI_ANALYTIC;
+            rtype = REGULARIZATION_BSPLINE_SEMI_ANALYTIC;
         }
         else if (val == "numeric") {
-            stage->regularization_type = REGULARIZATION_BSPLINE_NUMERIC;
+            rtype = REGULARIZATION_BSPLINE_NUMERIC;
         }
         else {
             goto error_exit;
         }
     }
-    else if (key == "regularization_lambda"
+    else if (key == "total_displacement_penalty") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.total_displacement_penalty) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "diffusion_penalty") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.diffusion_penalty) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "curvature_penalty"
+        || key == "regularization_lambda"
         || key == "young_modulus") {
         if (!section_stage) goto key_only_allowed_in_section_stage;
-        if (sscanf (val.c_str(), "%f", &stage->regularization_lambda) != 1) {
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.curvature_penalty) != 1) {
+            goto error_exit;
+        }
+#if PLM_CONFIG_LEGACY_SQUARED_REGULARIZER
+        stage->regularization_parms.curvature_penalty
+            *= stage->regularization_parms.curvature_penalty;
+#endif
+    }
+    else if (key == "curvature_mixed_weight") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.curvature_mixed_weight) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "lame_coefficient_1") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.lame_coefficient_1) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "lame_coefficient_2") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.lame_coefficient_2) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "linear_elastic_multiplier") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.linear_elastic_multiplier) != 1) {
+            goto error_exit;
+        }
+    }
+    else if (key == "third_order_penalty") {
+        if (!section_stage) goto key_only_allowed_in_section_stage;
+        if (sscanf (val.c_str(), "%f",
+                &stage->regularization_parms.third_order_penalty) != 1) {
             goto error_exit;
         }
     }
