@@ -103,6 +103,12 @@ save_ss_img (
         seg->save_ss_list (parms->output_ss_list_fn);
     }
 
+    /* Opt4D images */
+    if (parms->output_opt4d_fn != "") {
+        lprintf ("save_ss_img: save_opt4d\n");
+        seg->save_opt4d (parms->output_opt4d_fn);
+    }
+
     /* prefix images */
     if (parms->output_prefix != "") {
         lprintf ("save_ss_img: save_prefix\n");
@@ -154,6 +160,9 @@ warp_and_save_ss (
     }
 
     Segmentation::Pointer seg = rt_study->get_segmentation();
+    if (!seg->have_structure_set ()) {
+        return;
+    }
 
     /* If we have need to create image outputs, or if we have to 
        warp something, then we need to rasterize the volume */
@@ -161,6 +170,8 @@ warp_and_save_ss (
        because we might need the labelmap */
     if (parms->output_labelmap_fn != ""
         || parms->output_ss_img_fn != ""
+        || parms->output_opt4d_fn != ""
+        || parms->output_study_dirname != ""
         || parms->xf_in_fn != ""
         || parms->output_prefix != "")
     {
@@ -298,7 +309,9 @@ rt_study_warp (Rt_study *rt_study, Plm_file_format file_type, Warp_parms *parms)
     else if (rt_study->have_segmentation() &&
         /* use the spacing of the structure set */
         rt_study->get_segmentation()->have_structure_set() && 
-        rt_study->get_segmentation()->get_structure_set()->have_geometry) {
+        rt_study->get_segmentation()->get_structure_set()->have_geometry)
+    {
+        lprintf ("Setting PIH from structure set spacing\n");
         pih.set_from_gpuit (
             rt_study->get_segmentation()->get_structure_set()->m_dim, 
             rt_study->get_segmentation()->get_structure_set()->m_offset, 
@@ -313,12 +326,14 @@ rt_study_warp (Rt_study *rt_study, Plm_file_format file_type, Warp_parms *parms)
     {
         /* we have structure set, but without geometry.  use 
            heuristics to find a good geometry for rasterization */
+        lprintf ("Setting PIH from rasterization heuristics");
         rt_study->get_segmentation()->find_rasterization_geometry (&pih);
     } else {
         /* use some generic default parameters */
         plm_long dim[3] = { 500, 500, 500 };
         float origin[3] = { -249.5, -249.5, -249.5 };
         float spacing[3] = { 1., 1., 1. };
+        lprintf ("Setting PIH from generic defaults\n");
         pih.set_from_gpuit (dim, origin, spacing, 0);
     }
 
@@ -442,31 +457,38 @@ rt_study_warp (Rt_study *rt_study, Plm_file_format file_type, Warp_parms *parms)
         /* Convert ss_img to cxt */
         lprintf ("Rt_study_warp: Convert ss_img to cxt.\n");
         seg->convert_ss_img_to_cxt ();
+        if (seg->have_structure_set()) {
+            
+            /* Delete empty structures */
+            if (parms->prune_empty) {
+                lprintf ("Rt_study_warp: Prune empty structures.\n");
+                seg->prune_empty ();
+            }
 
-        /* Delete empty structures */
-        if (parms->prune_empty) {
-            lprintf ("Rt_study_warp: Prune empty structures.\n");
-            seg->prune_empty ();
-        }
-
-        /* Set the DICOM reference info -- this sets the internal geometry 
-           of the ss_image so we rasterize on the same slices as the CT? */
-        lprintf ("Rt_study_warp: Apply dicom_dir.\n");
-        seg->apply_dicom_dir (rt_study->get_rt_study_metadata());
+            /* Set the DICOM reference info -- this sets the internal geometry 
+               of the ss_image so we rasterize on the same slices as the CT? */
+            lprintf ("Rt_study_warp: Apply dicom_dir.\n");
+            seg->apply_dicom_dir (rt_study->get_rt_study_metadata());
         
-        /* Set the output geometry */
-        lprintf ("Rt_study_warp: Set geometry from PIH.\n");
-        seg->set_geometry (&pih);
+            /* Set the output geometry */
+            lprintf ("Rt_study_warp: Set geometry from PIH.\n");
+            seg->set_geometry (&pih);
 
-        /* Set rasterization geometry */
-        lprintf ("Rt_study_warp: Set rasterization geometry.\n");
-        seg->get_structure_set()->set_rasterization_geometry ();
+            /* Set rasterization geometry */
+            lprintf ("Rt_study_warp: Set rasterization geometry.\n");
+            seg->get_structure_set()->set_rasterization_geometry ();
+        }
     }
 
-    /* Warp and save structure set (except dicom) */
+    /* Warp and save structure set (except dicom and study formats) */
     lprintf ("Rt_study_warp: warp and save ss.\n");
     warp_and_save_ss (rt_study, xform, &pih, parms);
 
+    /* Save study format */
+    if (parms->output_study_dirname != "") {
+        rt_study->save (parms->output_study_dirname);
+    }
+
     /* Save dicom */
     if (parms->output_dicom != "") {
         lprintf ("Rt_study_warp: Save dicom.\n");
